package orchid.agent;

import orchid.object.*;
import java.util.*;
import java.util.Calendar;
import java.io.*;
import com.objectspace.voyager.mobility.*;
import com.objectspace.voyager.*;
import com.objectspace.lib.*;
import com.objectspace.lib.util.*;
import com.objectspace.lib.facets.*;
import com.objectspace.voyager.agent.*;

/**
 * The MeetingAgent class.<P>
 * To generate the interface : <TT> $ igen -v -d . orchid.agent.MeetingAgent</TT>.<P>
 * If something goes wrong, IMobility has to be <b>DE-implemented</b> since 
 * there may be a bug in this Interface.<BR>
 * If so, swap the 2 lines of declaration of the class and comment the 2 moveTo methods.
 * @author Marc-Aurle DARCHE & Antoine SOULIEZ
 */
//public class MeetingAgent implements IMobile, IMeetingAgent, Serializable {
public class MeetingAgent implements IMeetingAgent, IMobile, IMobility, Serializable {

	UserInfo agent_user;
	private String currentlocation;
	private Hashtable meetingProposalList = new Hashtable();
	Hashtable dateHourProposalList = new Hashtable();	//used by MAEngine02
	//static int REPLY_DELAY = 24*60;	//delay in minutes
	static int REPLY_DELAY = 2;		//delay in minutes
	
	/**
	 * boolean flag to tell what, Antoine ??? .
	 */
	boolean answerarrived = false;	//used by MAEngine01
	
	/**
	 * boolean flag to tell wether the agent has received all the other agents CF.
	 */
	int numofcfreceived = 0;	//used by MAEngine02
	
	/**
	 * boolean flag to tell wether the agent has been contacted by the previous MCP.
	 */
	boolean kMCP_contacted= false;	//used by MAEngine02
	boolean next= false;
	boolean nomore= false;

/**
 * MeetingAgent constructor
 */
	public MeetingAgent(UserInfo u){
		this.agent_user= u;
		Agent.of(this).setAutonomous(true);
		this.currentlocation = this.agent_user.getHome();
		System.out.println("I'm a newly created Agent. I am autonomous, and I belong to "+this.agent_user.getName()+" !");
	}

	public void finalize(){
		System.out.println("finalized MeetingAgent");
	}

	public void preDeparture( String source, String destination ){
		System.out.println(" preDeparture( " + source + ", " + destination + " )");
	}

	public void preArrival(){
		System.out.println(" preArrival() MeetingAgent");
	}

	public void postArrival(){
		System.out.println(" postArrival() MeetingAgent");
	}

	public void postDeparture(){
		System.out.println(" postDeparture() MeetingAgent");
	}

	public void moveTo (Object obj){
		System.out.println(" -- MeetingAgent: I'm moving ! ");
	}

	public void moveTo (String url){
		System.out.println(" -- MeetingAgent: I'm moving ! ");
	}

/**
 * Make the agent send the schedule to its client owner.<BR>
 * Client to Agent Method.
 */
	public Vector getMeetings(Date beginning, Date end) {
		ISaa saa;
		try {
			saa = (ISaa) Namespace.lookup("//"+this.agent_user.getHome()+"/SAA");
			return saa.getMeetings(this.agent_user.getName(),beginning,end);
		} catch(Exception e) {
			System.err.println("<ERROR from ma__"+this.agent_user.getName()+" > "+e);
		  }
		return null;
	}

/** 
 * Give the ma agents of the owners.
 */
	Vector getAgentsUrl(Vector owners){
		ISaa saa;
		Vector agentlist = new Vector();
		Vector agentlocal = new Vector();
		Vector serverlist = new Vector();
		int numberofagents=0;

		try{
			saa = (ISaa) Namespace.lookup("//"+this.currentlocation+"/SAA");
			serverlist = saa.getServers("");
		} catch (NamespaceException nsex) {
			System.err.println("<ERROR from ma__"+this.agent_user.getName()+" > Cannot find SAA");
			System.exit(1);
		}
		for (int j = 0 ; j < serverlist.size() ; j++){
			try
				{
					saa = (ISaa) Namespace.lookup("//"+((ServerInfo) serverlist.elementAt(j)).getUrl()+"/SAA");
					/* get all the meeting agent of the server contacted */
					agentlocal = saa.getAgents("where name like 'ma__%' and owner in "+Util.vectorToSQLList(owners));
				} catch (NamespaceException nsex2)
					 {
							System.err.println("<ERROR from ma__"+this.agent_user.getName()+" > Cannot find SAA "+((ServerInfo) serverlist.elementAt(j)).getUrl());
							System.exit(1);
					 }
					 
			for (int k = 0 ; k< agentlocal.size() ; k++) {
				agentlist.addElement(agentlocal.elementAt(k));
			}
		}
		return agentlist;
	}

/**
 * Make the Initiator Agent send the MeetingProposal to all the meeting guests' agents.<BR>
 * <b>Method used by the Initiator Agent</b>.
 */
	public void callMeeting(MeetingProposal mp){
		IMeetingAgent ma;
		Calendar endtime = Calendar.getInstance();

		orchid.object.Util.logMsg("info","ma__"+agent_user.getName(),"Call meeting");
		this.meetingProposalList.put(mp.getKey(),mp);
		
		// This test sets the answer of the initiator agent to 1, meaning it is coming to its own meeting (optimization).
		if ( agent_user.getName().equals(mp.getInitiator()) ) { 
			GuestInfo gi= (GuestInfo) mp.getGuestInfoList().get(agent_user.getName());
			gi.setAnswer(1);
		}

		//endtime.add(endtime.DATE,1);
		endtime.add(endtime.SECOND,8);
		int rythm= 1000;
		MAEngine01 mae01 = new MAEngine01(this, (MeetingProposal) meetingProposalList.get(mp.getKey()), endtime, rythm);
		mae01.start();
		Vector agentslist = getAgentsUrl(Util.enumToVector(mp.getGuestInfoList().keys()));

		for (int i = 0 ; i < agentslist.size() ; i++)
			try {
				AgentInfo agcontacted = (AgentInfo) agentslist.elementAt(i);
				// This test avoid the initiator agent to call a new meeting on itself (optimization).
				if ( ! agcontacted.getName().equals("ma__"+mp.getInitiator()) ) { 
					ma = (IMeetingAgent) Namespace.lookup("//"+agcontacted.getCurrentlocation()+"/"+agcontacted.getName());
					ma.newMeeting(mp);
				}
			} catch(Exception e) {
				orchid.object.Util.logMsg("error","ma__"+agent_user.getName(),"Can't reach the "+((AgentInfo) agentslist.elementAt(i)).getName()+" >> "+e);
			  }
	}

/**
 * Make the Agent store the MeetingProposal and ask its user if it will come or not to the Meeting.<BR>
 * <b>In the next step</b>, the Agent will have to use the its willComeOrNot method 
 * to transfer the answer to the Initiator Agent.<BR>
 * Agent to Agent Method.<BR>
 * <b>Method called by the Initiator Agent callMeeting method</b>.
 */
	public void newMeeting(MeetingProposal mp) {
		orchid.object.Util.logMsg("info","ma__"+agent_user.getName(),"New meeting");
		String initiatorName=mp.getInitiator();
		String initiator_maServer= "";
		Vector owner= new Vector();
		owner.addElement(initiatorName);
		initiator_maServer= ((AgentInfo) getAgentsUrl(owner).elementAt(0)).getCurrentlocation();
		IMeetingAgent initiator_ma;
		//this.meetingProposalList.addElement(mp);
		this.meetingProposalList.put(mp.getKey(),mp);
		boolean answer=true;
		// answer=askUser(mp); ???? Thread ???
		try {
			initiator_ma = (IMeetingAgent) Namespace.lookup("//"+initiator_maServer+"/ma__"+initiatorName);
			try {
				 Thread.currentThread().sleep(2000);
			} catch (Exception e) { System.err.println(e); }

			initiator_ma.willComeOrNot(mp.getKey(),this.agent_user.getName(),answer);
		} catch(Exception e) {
			orchid.object.Util.logMsg("error","ma__"+agent_user.getName(),"Can't reach the ma__"+initiatorName+" on server "+initiator_maServer+" >> >"+e);
		  }
	}

/**
 * Make the agent answer if it will come or not to a meeting.<BR>
 * Agent to Agent Method.
 */
	public void willComeOrNot(String meetingProposalKey, String userName, boolean answer) {
		orchid.object.Util.logMsg("info","ma__"+agent_user.getName()+" (willComeOrNot)","Answer from "+userName);
		MeetingProposal mp= (MeetingProposal) meetingProposalList.get(meetingProposalKey);
		orchid.object.Util.logMsg("info","ma__"+agent_user.getName()+" (willComeOrNot)","MP >> "+mp.toString());
		try
			{
				GuestInfo gi= (GuestInfo) mp.getGuestInfoList().get(userName);
				if (answer)
					gi.setAnswer(1);
				else
					gi.setAnswer(0);
				answerarrived = true;
			} catch (Exception exceptfornothing)
				{
					orchid.object.Util.logMsg("info","ma__"+agent_user.getName()+" (willComeOrNot)",userName+" is not in the MP "+meetingProposalKey+" any more");
				}
	}

/**
 * Make the agents of the participants update their meetingProposalList.<BR>
 * Agent to Agent Method.<BR>
 * <b>Method called by the Initiator Agent</b>.
 */
	public void updateMeeting(MeetingProposal mp) {
		orchid.object.Util.logMsg("info","ma__"+agent_user.getName()+" (updateMeeting)","update of the meeting "+mp.getKey());
		this.meetingProposalList.remove(mp.getKey());
		this.meetingProposalList.put(mp.getKey(),mp);
	}

	public void startGivingCF (String meetingProposalKey) {
		Calendar endtime = Calendar.getInstance();
		endtime.add(endtime.SECOND,8);
		int rythm= 1000;
		MAEngine02 mae02 = new MAEngine02(this, (MeetingProposal) meetingProposalList.get(meetingProposalKey), endtime, rythm);
		mae02.start();
	}

/**
 * Make the agent store the Constraint Factor (CF) of an agent's user.<BR>
 */
	public void giveYouMyCF (String meetingProposalKey, int CF, String username) {
		try {
			Thread.currentThread().sleep((long) Math.floor(Math.random() * 5000));
		} catch (Exception e) { System.err.println(e); }
		orchid.object.Util.logMsg("info","ma__"+agent_user.getName()+" (giveYouMyCF)","received CF from "+username);
		MeetingProposal mp = (MeetingProposal) this.meetingProposalList.get(meetingProposalKey);
		GuestInfo gi= (GuestInfo) mp.getGuestInfoList().get(username);
		gi.setCF(CF);
		numofcfreceived++;
	}

/**
 * Make the agent send a new DateHourProposal to the next most constrained participant's agent.<BR>
 * Agent to Agent Method.
 */
	public synchronized void newProposal(String meetingProposalKey, DateHourProposal dhp) {
		dateHourProposalList.put(meetingProposalKey, dhp);
		orchid.object.Util.logMsg("info","ma__"+agent_user.getName()+" (newProposal)","received DateHourProposal "+dhp);
		kMCP_contacted= true;
	}

/**
 * Agent to Agent Method.
 */
	public synchronized void nextProposal(String meetingProposalKey) {
		this.next=true;
	}

/**
 * Agent to Agent Method.
 */
	public synchronized void noMoreProposal(String meetingProposalKey) {
		this.nomore=true;
		System.out.println("<"+agent_user.getName()+">>>>>>>>>>>>>>> nomore toggled to "+nomore);
	}

/**
 * Make the agent go to its home server.<BR>
 * Agent to Agent Method.
 */
	public void goHome() {
		try{
			Agent.of(this).moveTo(this.agent_user.getHome());
		} catch (Exception e) {System.err.println(e);}
	}

/**
 * Make the agent give its RangeProposal to the initiator agent.<BR>
 * To perform this action the agent has to be the nMCP.<BR>
 * Agent to Agent Method.
 * <b>Method called onto the Initiator Agent</b>.
 */
	public void giveResult(String meetingProposalKey, RangeProposal rp) {
	}

/**
 * Make the agent confirm the final DateHourProposal to all the agents of one subgroup.<BR>
 * This is the LAST METHOD to be used.<BR>
 * Agent to Agent Method.<BR>
 * <b>Method called by the nMCP agent</b>.
 */
	public void confirmMeeting(MeetingProposal mp) {
		dateHourProposalList.remove(mp.getKey());
		//Meeting m = new Meeting(mp);
		Meeting m = new Meeting();
		try {
			ISaa saa = (ISaa) Namespace.lookup("//"+this.agent_user.getHome()+"/SAA");
			saa.addMeeting(agent_user.getName(),m);
		} catch(Exception e) {
			orchid.object.Util.logMsg("error","MeetingAgent",""+e);
		  }
	}

/**
 * Make the agent return the name of the server on which it was created.<BR>
 * Agent to Agent Method.
 */
	public String getHome(){
		return this.agent_user.getHome();
	}

/**
 * Make the agent return the name of the server on which it is running .<BR>
 */
	public String getCurrentlocation(){
		return this.currentlocation;
	}

/**
 * Make the agent return the name of its owner.<BR>
 * Agent to Agent Method.
 */
	public String getOwner(){
		return this.agent_user.getName();
	}

/**
 * Make the agent display a string at its current location.<BR>
 * For test/debug purpose.
 */
	public void print(String str){
		System.out.println("MeetingAgent from "+getOwner()+" prints "+str);
	}

/*************** ADDED FOR ENGINE 01 ***************/

/**
 * Test if everybody answered to the MeetingProposal.<BR>
 */
	public boolean isEverybodyAnswered(String meetkey)
		{
			int totalanswers = 0;

			Hashtable guestlist = ((MeetingProposal) meetingProposalList.get(meetkey)).getGuestInfoList();
			int totalguests = guestlist.size();

			Enumeration e = guestlist.keys();
			while (e.hasMoreElements())
				{
					if (((GuestInfo) guestlist.get(e.nextElement())).getAnswer() != -1)
						totalanswers ++;
				}
			return (totalanswers == totalguests);
		}
}

