package orchid.object;

import java.util.*;

/**
 * A class used to store meetings in a DataBase.<P>
 * A stored meeting looks like:<P>
 * <TT>
 * runion tped dfinition du terme agent ma_darche Thu Oct 01 00:00:00 GMT+00:00 1998 1200 400 philippe_benhamou emmanuel_pierre antoine_souliez true
 * </TT>
 * <ol>
 * <li> MeetingProposal
 * <li> DateHourProposal
 * <li> RangeProposal
 * <li> Meeting
 * </ol>
 * @see MeetingProposal
 * @see DateHourProposal 
 * @see RangeProposal
 * @see ObjectTest
 * @author Marc-Aurle DARCHE
 */
public class Meeting implements java.io.Serializable{

	private String subject;
	private String note;
	private String initiator;
	private Date aDate;		// yyyy-mm-dd
	private int timeOfDay;		// hh30 or hh00
	private int duration;		// hh30 or hh00
	private Vector participantList;
	boolean frozen;

	public Meeting() {
	}

	public Meeting(String subject, String note, String initiator, int year, int month, int day, int timeOfDay, int duration, Vector participantList, boolean frozen) {
		this.subject=subject;
		this.note=note;
		this.initiator=initiator;
		Calendar rightNow = Calendar.getInstance();
		rightNow.set(year,month,day,0,0,0);
		this.aDate=rightNow.getTime();
		this.timeOfDay=timeOfDay;
		this.duration=duration;
		this.participantList= participantList;
		this.frozen= frozen;
	}

	public void setNote(String note) {
		this.note=note;
	}

	public void setDate(int year, int month, int day) {
		Calendar rightNow = Calendar.getInstance();
		rightNow.set(year,month,day,0,0,0);
		this.aDate=rightNow.getTime();
	}

	public void setTimeOfDay(int timeOfDay) {
		this.timeOfDay=timeOfDay;
	}

	public void setFrozen(boolean state) {
		this.frozen= state;
	}

	public String getSubject() {
		return this.subject;
	}

	public String getNote() {
		return this.note;
	}

	public String getInitiator() {
		return this.initiator;
	}

	public Date getDate() {
		return this.aDate;
	}

	public int getTimeOfDay() {
		return this.timeOfDay;
	}

	public int getDuration() {
		return this.duration;
	}

	public Vector getParticipants() {
		return this.participantList;
	}

	public boolean getFrozen() {
		return this.frozen;
	}

	public String toString() {
		return subject+" "+note+" "+initiator+" "+
			aDate.toString()+" "+new Integer(timeOfDay).toString()+" "+
			new Integer(duration).toString()+" "+participantList+" "+frozen;
	}
	
}

