package orchid.object;

import java.util.*;
import java.io.*;
import java.math.*;

/**
 * A class used to propose the initiator agent a range of 
 * possible time slots for the meeting.<BR>
 * Time slots range from the first to the last proposed dates
 * for the meeting.<BR>
 * A range proposal looks like:<P>
 * <TT>
 * 0 0 0 1020 1044480 0 0 0 0 16320 0 0
 * </TT>
 * <P>The objects to arrange a meeting are used in this order :
 * <ol>
 * <li> MeetingProposal
 * <li> DateHourProposal
 * <li> RangeProposal
 * <li> Meeting
 * </ol>
 * @see MeetingProposal
 * @see DateHourProposal
 * @see Meeting
 * @see ObjectTest
 * @author Marc-Aurle DARCHE
 */
public class RangeProposal implements java.io.Serializable{

	private int range[];

	public RangeProposal() {
	}

	/**
	 * The ranges are from 800 (08H00) to 2200 (22H00) and last 30 minutes.
	 * ( 22 - 8 ) x 2 = 14 x 2 = 28 <BR>
	 * The range proposal is initialized with zeros and is later filled 
	 * with numbers corresponding to DateHourProposals.
	 */
	public RangeProposal(int numOfDays) {
		System.out.println("RangeProposal numOfDays : "+numOfDays);
		this.range= new int[numOfDays];
		for (int i = 0; i < numOfDays ; i++) {this.range[i]=0;}
	}

	public void setMeeting(Date fromDate, DateHourProposal dhp, int duration) throws IOException {
		Calendar fromCal = Calendar.getInstance();
		Calendar diffCal = Calendar.getInstance();
		fromCal.setTime(fromDate);
		diffCal.setTime(dhp.getDate());
		diffCal.set(Calendar.HOUR_OF_DAY, (dhp.getTimeOfDay()) / 100 );
		diffCal.set(Calendar.MINUTE, (dhp.getTimeOfDay()) % 100 );

		//if ( ! diffCal.equals(fromCal) && diffCal.before(fromCal) ) {
		if ( diffCal.before(fromCal) ) {
			System.err.println(" RangeProposal Exception :");
			System.err.println("  -> DateHourProposal date is earlier than the RangeProposal");
			System.exit(-1);
		}

		int tablePlace= -1; // number of days since the begining of the range
		while (diffCal.after(fromCal)) {
			diffCal.add(Calendar.DATE,-1); tablePlace++;
		}
		//System.out.println("place after first day : "+tablePlace);
		int appointmentTime = dhp.getTimeOfDay();
		//System.out.println("appointmentTime : "+appointmentTime);
		int dhpHourInTable= appointmentTime - 800;
		int fromTimeOfDay= (dhpHourInTable / 100) * 2 + (dhpHourInTable % 100) / 30;
		//System.out.println("fromTimeOfDay : "+fromTimeOfDay);

		//System.out.println("duration : "+duration);
		// meeting duration by slots of 30 min
		// "/ = div , % = mod"
		int durationMod30 = ((duration / 100) * 2) + ((duration % 100) / 30);
		//System.out.println("durationMod30 : "+durationMod30);
		BigInteger two= new BigInteger("2");
		for (int c = fromTimeOfDay ; c < (fromTimeOfDay + durationMod30) ; c++) {
			//System.out.println("c : "+c+" ");
			range[tablePlace]+= two.pow(c).intValue();
		}
	}

	public int[] getRange() {
		return this.range;
	}

	public int getDayRange(int n) {
		return this.range[n];
	}

	public int getRangeSize() {
		return this.range.length;
	}

	public String toString() {
		String displayString= "";
		for (int i = 0; i < this.range.length; i ++) {
				displayString = displayString+new Integer(this.range[i]).toString()+" ";
		}
		return displayString;
	}

	static public void main (String args[]){
		Calendar fromCal = Calendar.getInstance();
		System.err.println("fromCal: "+fromCal.getTime());
		//Calendar diffCal = Calendar.getInstance();
		Calendar diffCal = fromCal;
		System.err.println("diffCal: "+diffCal.getTime());
		if (diffCal.before(fromCal)) {
			// <=
			System.err.println(" 1");
		} else {
			// <
			System.err.println(" 2");
		}
		BigInteger bi= new BigInteger("2");
		Integer i=new Integer(  (bi.pow(29)).intValue() );
		System.out.println("2^29 = "+i.toString()  );
		System.out.println("Original Number: 530");
		System.out.println("530 Div 100 = "+ 530 / 100  );
		System.out.println("530 Mod 100 = "+ 530 % 100  );
	}

}
