package orchid.server;

import java.io.*;
import java.util.*;
import java.util.Date;
import com.objectspace.voyager.*;
import com.objectspace.voyager.space.*;
import com.objectspace.voyager.space.Subspace.*;
import com.objectspace.voyager.agent.*;
import com.objectspace.lib.*;
import com.objectspace.lib.util.*;

/** Tracer is the server logfile manager
	* It is used by the server and his Saa to keep a trace of all events
	*/
public class Tracer
	{
    private String selfID;
		private String LOGFILENAME = "Serverlog";
		private char sysseparator = '/';
		private boolean allowed;
		private RandomAccessFile raflog;

		public Tracer(String pathoflogfile, String serverID)
			{
				this.selfID = serverID;
				this.LOGFILENAME = this.LOGFILENAME +"_"+ selfID;

				SecurityManager security = System.getSecurityManager();
				if (security != null)
					try
						{
							security.checkWrite(pathoflogfile);
							allowed = true;
						} catch (SecurityException se1)
							{
								System.err.println("<ERROR Tracer "+selfID+" > Cannot create logfile in "+pathoflogfile+" - Permission denied");
								System.err.println("All event trace will be lost !");
								allowed = false;
							}
					else
						{
							System.err.println("<ALERT from Tracer "+selfID+" > No Security Manager on this system");
						}
				
				allowed=true;
				if (allowed)
					{
						File logfile = new File(pathoflogfile+sysseparator+LOGFILENAME);
						while (logfile.exists())
							{
								LOGFILENAME = LOGFILENAME+"#";
								logfile = new File(pathoflogfile+sysseparator+LOGFILENAME);
							}
						System.out.println("[info from Tracer "+selfID+" ] logfile is "+LOGFILENAME);
						try
							{
								raflog = new RandomAccessFile(pathoflogfile+sysseparator+LOGFILENAME,"rw");

							} catch (IOException ioe1)
								{
									System.err.println("<ERROR from Tracer "+selfID+" > Cannot create logfile "+LOGFILENAME + " : "+ioe1.toString());
								System.err.println("All event trace will be lost !");
								allowed = false;
									
								}
						System.out.println("[info from Tracer "+selfID+" ] Tracer initialized");
					}
			}
	
	public void logMessage(String purpose, String identity, String messagetolog)
		{
			try
				{
					if (((purpose.toLowerCase()).equals("error")) ||
							((purpose.toLowerCase()).equals("panic")) ||
							((purpose.toLowerCase()).equals("alert")))
						raflog.writeBytes((new Date(System.currentTimeMillis())).toString()+": <"+purpose.toUpperCase()+" from "+identity+"> "+messagetolog+"\n");
					else
						raflog.writeBytes((new Date(System.currentTimeMillis())).toString()+": ["+purpose+" from "+identity+"] "+messagetolog+"\n");
				} catch (IOException ioe2)
					{
						System.err.println("<ERROR from Tracer "+selfID+" > Cannot write message down");
					}
		}
	
	public void die()
		{
			try
				{
					raflog.close();
				} catch (IOException ioe3)
					{
						System.err.println("<ERROR from Tracer "+selfID+" > Cannot close de logfile correctly");
					}
		}

	public static void main (String[] args)
		{
			if (args.length != 3)
				{
					System.out.println("Use 2 arguments :\n - path of the logfile (without last separator\n - Identity of the server\n");
					System.exit(0);
				}
			Tracer tracer = new Tracer(args[0],args[1]);
			tracer.logMessage("info","nobody","Je suis la et c'est bien !");
			tracer.logMessage("alert","nobody","Je suis encore la !");
			tracer.die();
		}
	}
